package gov.nih.nlm.nls.lvg.Db;
import gov.nih.nlm.nls.lvg.Lib.*;
/*****************************************************************************
* This class defined the data structure of a derivation record.
*
* <p><b>History:</b>
*
* @author NLM NLS Development Team
*
* @see 
* <a href="../../../../../../../designDoc/UDF/database/derivationTable.html">
* Desgin Document </a>
*
* @version    V-2010
****************************************************************************/
public class DerivationRecord
{
    // public constructor
    /**
    * Create a derivation record, using source term, source category,
    * target term, and target category.
    */
    public DerivationRecord(String source, int sourceCat, String target, 
        int targetCat)
    {
        source_ = source;
        target_ = target;
        sourceCat_ = sourceCat;
        targetCat_ = targetCat;
    }
    // public methods
    /**
    * Get the string value of source term from current derivation record.
    *
    * @return  the string value of source term from current derivation record
    */
    public String GetSource()
    {
        return source_;
    }
    /**
    * Get the string value of target term from current derivation record.
    *
    * @return  the string value of target term from current derivation record
    */
    public String GetTarget()
    {
        return target_;
    }
    /**
    * Get the source category from current derivation record.
    *
    * @return  integer value of source category from current derivation record
    */
    public int GetSourceCat()
    {
        return sourceCat_;
    }
    /**
    * Get the target category from current derivation record.
    *
    * @return  integer value of target category from current derivation record
    */
    public int GetTargetCat()
    {
        return targetCat_;
    }
    /**
    * Get the whole record in a string format.  The format is
    * <br>source term | source cateoory | target term | target category
    * <br> where categories are integers
    *
    * @return  the string value of whole current derivation record
    */
    public String GetString(String sp)
    {
        String out = source_ + sp + sourceCat_ + sp + target_ + sp 
            + targetCat_ + sp;
        return out;
    }
    /**
    * Get the whole record in a pure string format.  The format is
    * <br>source term | source cateoory | target term | target category
    * <br> where categories are in string names.
    *
    * @return  the string value of whole current derivation record
    */
    public String GetPureString(String sp)
    {
        String out = source_ + sp + Category.ToName(sourceCat_) + sp 
            + target_ + sp + Category.ToName(targetCat_) + sp;
        return out;
    }
    // data member
    private String source_ = null;        // source: input term
    private String target_ = null;        // target: output term
    private int sourceCat_ = 0;           // source Category 
    private int targetCat_ = 0;           // target Category 
}
