import java.util.*;
import java.sql.*;
import java.io.*;
import gov.nih.nlm.nls.lvg.CmdLineSyntax.*;
import gov.nih.nlm.nls.lvg.Lib.*;
import gov.nih.nlm.nls.lvg.Util.*;
import gov.nih.nlm.nls.lvg.Flows.*;
import gov.nih.nlm.nls.lvg.Db.*;
import gov.nih.nlm.nls.lvg.Trie.*;
/*****************************************************************************
* This class is the command line program for Norm
*
* <p><b>History:</b>
* <ul>
* <li>SCR-6, chlu, 08-05-09, update Lexical Tool version to 2010
* </ul>
*
* @author NLM NLS Development Team
*
* @version    V-2010
****************************************************************************/
public class norm extends SystemOption
{
    // public method
    // The flow chart for user-interface are:
    // main() -> Norm.ExecuteCommands: to setup flags
    // main() -> Norm.GetVariants() -> LineHandler()
    public static void main(String[] args)
    {
        String optionStr = "";
        Option io = new Option(optionStr);
        // form optin String if there are arguments
        if(args.length > 0)
        {
            optionStr = "";
            for(int i = 0; i < args.length; i++)
            {
                if(i == 0)
                {
                    optionStr = args[i];
                }
                else
                {
                    optionStr += (" " + args[i]);
                }
            }
            io = new Option(optionStr);
        }
        // define the system option flag & argument
        norm norm = new norm();
        // execute command according to option & argument
        if(SystemOption.CheckSyntax(io, norm.GetOption(), false, true) == true)
        {
            // decode input option to form options
            norm.ExecuteCommands(io, norm.GetOption());
            // get config file from environment variable
            boolean useClassPath = false;
            if(configFile_ == null)
            {
                useClassPath = true;
                configFile_ = "data.config.lvg";
            }
            // read in configuration file
            if(conf_ == null)
            {
                conf_ = new Configuration(configFile_, useClassPath);
            }
            if(maxTerm_ == -1)
            {
                maxTerm_ = Integer.parseInt(
                    conf_.GetConfiguration(Configuration.MAX_UNINFLS));
            }
            if(conf_.GetConfiguration(Configuration.NO_OUTPUT) != null)
            {
                noOutputStr_ = conf_.GetConfiguration(Configuration.NO_OUTPUT);
            }
            if(stopWords_ == null)
            {
                stopWords_ = ToStripStopWords.GetStopWordsFromFile(conf_);
            }
            if(symbolMap_ == null)
            {
                symbolMap_ = ToMapSymbolToAscii.GetSymbolMapFromFile(conf_);
            }
            if(unicodeMap_ == null)
            {
                unicodeMap_ = ToMapUnicodeToAscii.GetUnicodeMapFromFile(conf_);
            }
            if(diacriticMap_ == null)
            {
                diacriticMap_ 
                    = ToStripDiacritics.GetDiacriticMapFromFile(conf_);
            }
            if(ligatureMap_ == null)
            {
                ligatureMap_ = ToSplitLigatures.GetLigatureMapFromFile(conf_);
            }
            if(nonStripMap_ == null)
            {
                nonStripMap_ = ToStripMapUnicode.GetNonStripMapFromFile(conf_);
            }
            if(removeSTree_ == null)
            {
                removeSTree_ = ToRemoveS.GetRTrieTreeFromFile(conf_);
            }
            if(Platform.IsWindow() == true)
            {
                promptStr_ = 
                "- Please input a term (type \"Ctl-z\" then \"Enter\" to quit) >";
            }
            else
            {
                promptStr_ = "- Please input a term (type \"Ctl-d\" to quit) >";
            }
            if(conf_.GetConfiguration(Configuration.LVG_PROMPT).equals(
                "DEFAULT") == false)
            {
                 promptStr_ = conf_.GetConfiguration(Configuration.LVG_PROMPT);
            }
            // read in input text and get the Norm for it
            if(runFlag_ == true)
            {
                norm.GetVariants();
            }
        }
        else
        {
            NormHelp.NormHelp(outWriter_, fileOutput_);
        }
        try
        {
            Close();
        }
        catch (Exception e)
        {
        }
    }
    // read data from user's line inputs
    public static void GetVariants()
    {
        // decide the input source: from screen or file
        if(inReader_ == null)
        {
            try
            {
                inReader_ = new BufferedReader(new InputStreamReader(
                    System.in, "UTF-8"));
            }
            catch (IOException e)
            {
                System.err.println("**Error: problem of reading std-in");
            }
        }
        int minTermLen = Integer.parseInt(
            conf_.GetConfiguration(Configuration.MIN_TERM_LENGTH));
        String lvgDir = conf_.GetConfiguration(Configuration.LVG_DIR);
        int minTrieStemLength = Integer.parseInt(
            conf_.GetConfiguration(Configuration.DIR_TRIE_STEM_LENGTH));
        // perform Norm operation
        try
        {
            conn_ = DbBase.OpenConnection(conf_);
            trie_ = new RamTrie(true, minTermLen, lvgDir, minTrieStemLength);
            while(true) // Loop forever - to go into prompt
            {
                if(promptFlag_ == true)
                {
                    GetPrompt();     // Display a prompt to the user
                }
                // Read a line from the input source
                if(LineHandler(inReader_.readLine()) == false)
                {
                    break;
                }
            }
            Close();    // close files & database
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    // protected methods
    protected void ExecuteCommand(OptionItem optionItem, Option systemOption)
    {
        OptionItem nameItem =
            OptionUtility.GetItemByName(optionItem, systemOption, false);
        Vector<OptionItem> systemItems = systemOption.GetOptionItems();
        if(CheckOption(nameItem, "-ci") == true)
        {
            try
            {
                // get config file from environment variable
                boolean useClassPath = false;
                String configFile = configFile_;
                if(configFile == null)
                {
                    useClassPath = true;
                    configFile = "data.config.lvg";
                }
                Configuration conf = conf_;
                if(conf == null)
                {
                    conf = new Configuration(configFile, useClassPath);
                }
                Out.Println(outWriter_, conf.GetInformation(), fileOutput_,
                    false);
            }
            catch (IOException e) { }
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-d") == true)
        {
            detailsFlag_ = true;
        }
        else if(CheckOption(nameItem, "-h") == true)
        {
            NormHelp.NormHelp(outWriter_, fileOutput_);
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-hs") == true)
        {
            systemOption.PrintOptionHierachy();        // not UTF-8
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-i:STR") == true)
        {
            String inFile = nameItem.GetOptionArgument();
            if(inFile != null)
            {
                try
                {
                    inReader_ = new BufferedReader(new InputStreamReader(
                        new FileInputStream(inFile), "UTF-8"));
                }
                catch (IOException e)
                {
                    runFlag_ = false;
                    System.err.println(
                        "**Error: problem of opening/reading file " + inFile);
                }
            }
        }
        else if(CheckOption(nameItem, "-n") == true)
        {
            noOutputFlag_ = true;
        }
        else if(CheckOption(nameItem, "-o:STR") == true)
        {
            String outFile = nameItem.GetOptionArgument();
            if(outFile != null)
            {
                try
                {
                    outWriter_ = new BufferedWriter(new OutputStreamWriter(
                        new FileOutputStream(outFile), "UTF-8"));
                    fileOutput_ = true;
                }
                catch (IOException e)
                {
                    runFlag_ = false;
                    System.err.println(
                        "**Error: problem of opening/writing file " + outFile);
                }
            }
        }
        else if(CheckOption(nameItem, "-p") == true)
        {
            promptFlag_ = true;
        }
        else if(CheckOption(nameItem, "-s:STR") == true)
        {
            separator_ = nameItem.GetOptionArgument();
            
            // if sub by tab
            if(separator_.equals("\\t"))
            {
                separator_ = new Character((char)(9)).toString();    // tab
            }
        }
        else if(CheckOption(nameItem, "-t:INT") == true)
        {
            termFieldNum_ = Integer.parseInt(nameItem.GetOptionArgument());
        }
        else if(CheckOption(nameItem, "-ti") == true)
        {
            filterInputFlag_ = true;
        }
        else if(CheckOption(nameItem, "-v") == true)
        {
            try
            {
                Out.Println(outWriter_, "norm.2010", fileOutput_, false);
            }
            catch (IOException e) { }
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-x:STR") == true)
        {
            configFile_ = nameItem.GetOptionArgument();
        }
    }
    protected void DefineFlag()
    {
        // define all option flags & arguments by giving a option string
        String flagStr = "-ci -d -h -hs -i:STR -n -o:STR -p -s:STR -t:INT -ti -v -x:STR";
        
        // init the system option
        systemOption_ = new Option(flagStr);
        // Add the full name for flags
        systemOption_.SetFlagFullName("-ci", "Print_Config_Info");
        systemOption_.SetFlagFullName("-d", "Print_Operation_Details");
        systemOption_.SetFlagFullName("-h", "Help");
        systemOption_.SetFlagFullName("-hs", "Hierarchy_Struture");
        systemOption_.SetFlagFullName("-i", "Input_File");
        systemOption_.SetFlagFullName("-n", "No_Output");
        systemOption_.SetFlagFullName("-o", "Output_File");
        systemOption_.SetFlagFullName("-p", "Show_Prompt");
        systemOption_.SetFlagFullName("-s", "Field_Separator");
        systemOption_.SetFlagFullName("-t", "Term_Field");
        systemOption_.SetFlagFullName("-ti", "Display_Filtered_Input");
        systemOption_.SetFlagFullName("-v", "Version");
        systemOption_.SetFlagFullName("-x", "Load_Configuration_file");
    }
    // private methods
    private static void Close() throws IOException, SQLException
    {
        if(outWriter_ != null)
        {
            outWriter_.close();
        }
        if(inReader_ != null)
        {
            inReader_.close();
        }
        if(conn_ != null)
        {
            DbBase.CloseConnection(conn_, conf_);      // close db connection
        }
    }
    private static void GetPrompt() throws IOException
    {
        Out.Println(outWriter_, promptStr_, fileOutput_, false);
    }
    private static boolean LineHandler(String line) 
        throws SQLException, IOException
    {
        // check input for quitiing
        if(line == null)    
        {
            return false;
        }
        // Get Norm result by going through flow:t:g:o:l:B:w
        // get the term from appropriate field
        String inTerm = In.GetField(line, separator_, termFieldNum_);
        LexItem in = new LexItem(inTerm);
        Vector<LexItem> result = ToNormalize.Mutate(in, maxTerm_, stopWords_, 
            conn_, trie_, symbolMap_, unicodeMap_, ligatureMap_, diacriticMap_, 
            nonStripMap_, removeSTree_, detailsFlag_, false);
        
        // print results
        // check no output flag
        if((noOutputFlag_ == true)    // no output flag
        && ((result.size() == 0)    // no output LexItem
         || ((result.size() == 1)     // one output lexItem with empty string
          && (result.elementAt(0).GetTargetTerm().length() == 0))))
        {
            String out = line;
            if(filterInputFlag_ == true)
            {
                out = in.GetOriginalTerm();
            }
            Out.Println(outWriter_, (out + separator_ + noOutputStr_), 
                fileOutput_, false);
        }
        else
        {
            for(int i = 0; i < result.size(); i++)
            {
                LexItem temp = result.elementAt(i);
                String out = line;
                if(filterInputFlag_ == true)
                {
                    out = temp.GetOriginalTerm();
                }
                Out.Println(outWriter_, (out + separator_ 
                    + temp.GetTargetTerm()), fileOutput_, false);
                // print details
                if(detailsFlag_ == true)
                {
                    Out.Println(outWriter_, temp.GetDetailInformation(), 
                        fileOutput_, false);
                }
            }
        }
        return true;
    }
    // data member
    private final static String NO_OUTPUT_STR = "-No Output-";
    private static String noOutputStr_ = NO_OUTPUT_STR;
    private static boolean detailsFlag_ = false;    // flag for details print
    private static boolean runFlag_ = true;         // flag for running Norm
    private static boolean noOutputFlag_ = false;   // flag for return no output
    private static boolean promptFlag_ = false;     // flag for display prompt
    private static boolean filterInputFlag_ = false;  // flag for filtered input
    private static Hashtable<Character, String> symbolMap_ = null;
    private static Hashtable<Character, String> unicodeMap_ = null;
    private static Hashtable<Character, Character> diacriticMap_ = null;
        // diacritic for q
    private static Hashtable<Character, String> ligatureMap_ = null;   
        // ligature for q2
    private static Hashtable<Character, String> nonStripMap_ = null;
    private static RTrieTree removeSTree_ = null;  // removeS tree, rs
    private static String separator_ = GlobalBehavior.GetFieldSeparator();
    private static BufferedReader inReader_ = null;         // infile buffer
    private static BufferedWriter outWriter_ = null;        // outfile buffer
    private static boolean fileOutput_ = false;        // flag for file output
    private static int termFieldNum_ = 1;
    private static Connection conn_ = null;      // database connection
    private static RamTrie trie_ = null;         // ram Trie
    private static String configFile_ = null;
    private static String promptStr_ = null;
    private static Configuration conf_ = null;
    private static int maxTerm_ = -1;
    private static Vector<String> stopWords_ = null;
    static
    {
        try
        {
            outWriter_ = new BufferedWriter(new OutputStreamWriter(
                System.out, "UTF-8"));
        }
        catch (IOException e)
        {
            System.err.println("**Error: problem of opening Std-out.");
        }
    }
}
